/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is tkishdemos.
 *
 * The Initial Developer of the Original Code is
 * M.D. Nauta.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


function initialize() {
	document.getElementById('rgbColors').setAttribute('rgb','192:192:192');
	var adjustedColorsRadiogroup = document.getElementById('adjustedColors');
	adjustedColorsRadiogroup.addEventListener('DOMAttrModified',setNextStorageFromSelectedItem,false);
	if (window.arguments[0]) 
		document.documentElement.setAttribute('title',window.arguments[0]);
}

function returnRGB() {
	// Function called in oncommand OK-button.
	var RGB = document.getElementById('rgbColors').getAttribute('rgb').split(':');
	for (var i=0; i<3; i++) RGB[i] = Math.round(RGB[i]);
	if (typeof(window.arguments[1]) == 'function') {
		window.arguments[1](RGB.join(','));
	}
	window.close();
}

function RGB2HSVms(RGB) {
	// input: 0<=r,g,b<=1 where r,g,b is an array or individual values
	// ouput: 0<=hue<=360, 0<=saturation<=1, 0<=value<=1
	var red, green, blue, hue, saturation, value, max, min, sum, delta;
	RGB = (typeof(RGB)=='object') ? RGB.slice(0,3) :
		[arguments[0],arguments[1],arguments[2]];
	red = parseFloat(RGB[0]);
	green = parseFloat(RGB[1]);
	blue = parseFloat(RGB[2]);
	max = Math.max(red,green,blue);
	min = Math.min(red,green,blue);
	sum = max + min;
	delta = max - min;
	value = 0.5*sum;
	saturation = (sum<=1 ? delta/sum : delta/(2-sum));
	if (isNaN(saturation)) saturation = 0;
	if (red == max) {
		hue = (green-blue)/delta;
	} else {
		if (green == max) {
			hue = 2 + (blue-red)/delta;
		} else {
			hue = 4 + (red-green)/delta;
		}
	}
	if (isNaN(hue)) hue = 4;
	hue *= 60;
	if (hue < 0) hue += 360;
	return [hue,saturation,value];
}

/*
function RGB2HSV(RGB) {
	// input: 0<=r,b,g<=1 where r,g,b is an array or individual values
	// output: 0<=hue<360, 0<=saturation<=1, 0<=value<=1
	var red, green ,blue, hue, saturation, value; //value = brightness
	RGB = (typeof(RGB)=='object') ? RGB.slice(0,3) :
		[arguments[0],arguments[1],arguments[2]];
	red = RGB[0];
	green = RGB[1];
	blue = RGB[2];
	value = Math.max(red,green,blue);
	var delta = value - Math.min(red,green,blue);
	if (value) {
		saturation = delta/value;
	} else {
		hue = parseInt('NaN');
		saturation = 0;
		return [hue,saturation,value];
	}
	if (red == value) {
		hue = (green-blue)/delta;
	} else {
		if (green == value) {
			hue = 2 + (blue-red)/delta;
		} else {
			hue = 4 + (red- green)/delta;
		}
	}
	hue *= 60;
	if (hue < 0)  hue += 360;
	return [hue, saturation, value];
}
*/
function HSV2RGBms(HSV) {
	// input: 0<=hue<=360, 0<=saturation<=1, 0<=value<=1
	// output: 0<=r,g,b<=1 
	var red, green, blue, hue, saturation, value
	HSV = (typeof(HSV)=='object') ? HSV.slice(0,3) :
		[arguments[0],arguments[1],arguments[2]];
	hue = parseFloat(HSV[0]);
	saturation = parseFloat(HSV[1]);
	value = parseFloat(HSV[2]);
	if (saturation == 0) return [value,value,value];
	hue /= 60;
	var hueRough = Math.floor(hue);
	switch(hueRough) {
	case 0: if (value<0.5) {
			red = value*(1+saturation);
			blue = value*(1-saturation);
		} else {
			red = value*(1-saturation) + saturation;
			blue = value*(1+saturation) - saturation;
		}
		green = blue + (red-blue)*hue;
		break;
	case 1: if (value<0.5) {
			green = value*(1+saturation);
			blue = value*(1-saturation);
		} else {
			green = value*(1-saturation) + saturation;
			blue = value*(1+saturation) - saturation;
		}
		red = blue - (hue-2)*(green-blue);
		break;
	case 2: if (value<0.5) {
			green = value*(1+saturation);
			red = value*(1-saturation);
		} else {
			green = value*(1-saturation) + saturation;
			red = value*(1+saturation) - saturation;
		}
		blue = red + (hue-2)*(green-red);
		break;
	case 3: if (value<0.5) {
			blue = value*(1+saturation);
			red =value*(1-saturation);
		} else {
			blue = value*(1-saturation) + saturation;
			red = value*(1+saturation) - saturation;
		}
		green = red - (hue-4)*(blue-red);
		break;
	case 4: if (value<0.5) {
			blue = value*(1+saturation);
			green = value*(1-saturation);
		} else {
			blue = value*(1-saturation) + saturation;
			green = value*(1+saturation) - saturation;
		}
		red = green + (hue-4)*(blue-green);
		break;
	default: if (value<0.5) {
			red = value*(1+saturation);
			green = value*(1-saturation);
		} else {
			red = value*(1-saturation) + saturation;
			green = value*(1+saturation) - saturation;
		}
		blue = green - (hue-6)*(red-green);
		break;
	}
	return [red,green,blue];
}

function updateHSVfromRGB() {
	var hsvBC = document.getElementById('hsvColors');
	var RGB = hsvBC.getAttribute('rgb').split(':');
	for (var i=0; i<3; i++) RGB[i] /= 255;
	var HSV = RGB2HSVms(RGB);
	for (var i=0; i<3; i++) HSV[i] *= 240;
	HSV[0] /= 360;
	hsvBC.setAttribute('hsv',HSV.join(':'));
}

function updateRGBfromHSV() {
	var treshold = 0.001; // Prevents broadcasters updating each other constantly.
	var rgbBC = document.getElementById('rgbColors');
	var HSV = rgbBC.getAttribute('hsv').split(':');
	var RGB_orig = rgbBC.getAttribute('rgb').split(':');
	for (var i=0; i<3; i++) HSV[i] /= 240;
	HSV[0] *= 360;
	var RGB = HSV2RGBms(HSV);
	for (var i=0; i<3; i++) RGB[i] *= 255;
	if (Math.abs(RGB_orig[0]-RGB[0]) > treshold ||
		Math.abs(RGB_orig[1]-RGB[1]) > treshold ||
		Math.abs(RGB_orig[2]-RGB[2]) > treshold) {
		rgbBC.setAttribute('rgb',RGB.join(':'));
	}
}

function colorButton2bc(clrStr) {
	var bc = document.getElementById('rgbColors');
	bc.setAttribute('rgb',clrStr);
}

function setSampleColor() {
	var sample = document.getElementById('sample');
	var rgb = sample.getAttribute('rgb').split(':');
	var red = Math.round(rgb[0]);
	var green = Math.round(rgb[1]);
	var blue = Math.round(rgb[2]);
	sample.setAttribute('style','background-color: rgb(' + red + ',' + 
		green + ',' + blue + ');');
}

function setHSVbc() {
	var hue, saturation, brightness;
	hue = document.getElementById('hueValue').value;
	saturation = document.getElementById('saturationValue').value;
	brightness = document.getElementById('brightnessValue').value;
	var hsvBC = document.getElementById('hsvColors');
	hsvBC.setAttribute('hsv',hue + ':' + saturation + ':' + brightness);
}

function setHSVvalues() {
	var hue = document.getElementById('hueValue');
	var saturation = document.getElementById('saturationValue');
	var brightness = document.getElementById('brightnessValue');
	var HSV = hue.getAttribute('hsv').split(':');
	hue.value = Math.round(HSV[0]);
	saturation.value = Math.round(HSV[1]);
	brightness.value = Math.round(HSV[2]);
}

function updateColor(index,value) {
	var bc = document.getElementById('rgbColors');
	var RGB = bc.getAttribute('rgb').split(':');
	RGB[index] = value;
	bc.setAttribute('rgb',RGB.join(':'));
}

function setRGBcolors() {
	var red = document.getElementById('redValue');
	var green = document.getElementById('greenValue');
	var blue = document.getElementById('blueValue');
	var RGB = red.getAttribute('rgb').split(':');
	red.value = Math.round(RGB[0]);
	green.value = Math.round(RGB[1]);
	blue.value = Math.round(RGB[2]);
}

function imageClicked(evt) {
	var image = document.getElementById('hueSaturation');
	var imageWidth = image.boxObject.width;
	var imageHeight = image.boxObject.height;
	var clickX = evt.clientX - image.boxObject.x;
	var clickY = imageHeight - evt.clientY + image.boxObject.y - 1; //I don''t know why the -1.
	//alert(clickX + ' ' + clickY + ' ' + imageHeight + ' ' + imageWidth);
	var hue = 240*clickX/(imageWidth-1);
	var saturation = 240*clickY/(imageHeight-1);
	var hsvBC = document.getElementById('hsvColors');
	var brightness = (hsvBC.getAttribute('hsv').split(':'))[2];
	hsvBC.setAttribute('hsv',hue + ':' + saturation + ':' + brightness);
}

function brightnessBarClicked(evt) {
	var bbar = document.getElementById('brightnessSlider');
	var bbarHeight = bbar.boxObject.height;
	var clickY = bbarHeight- evt.clientY + bbar.boxObject.y - 1;
	var brightness = 240*clickY/(bbarHeight-1);
	var hsvBC = document.getElementById('hsvColors');
	var HSV = hsvBC.getAttribute('hsv').split(':');
	HSV[2] = brightness;
	hsvBC.setAttribute('hsv',HSV.join(':'));
}

function putHScursor() {
	var cursor = document.getElementById('HScursor');
	var cursorHalfWidth = Math.floor(cursor.boxObject.width/2);
	var cursorHalfHeight = Math.floor(cursor.boxObject.height/2);
	var HSV = cursor.getAttribute('hsv').split(':');
	for (var i=0; i<3; i++) HSV[i] /= 240;
	var HSimage = document.getElementById('hueSaturation');
	var HSimageWidth = HSimage.boxObject.width;
	var HSimageHeight = HSimage.boxObject.height;
	HSimageWidth -= 1; // Start count at 0 instead of 1.
	HSimageHeight -= 1;
	cursor.setAttribute('left',Math.round(HSV[0]*HSimageWidth) 
		- cursorHalfWidth);
	cursor.setAttribute('top',Math.round((1-HSV[1])*HSimageHeight) 
		- cursorHalfHeight);
}

function adjustBrightnessSlider() {
	var bslider = document.getElementById('brightnessSlider');
	var HSV = bslider.getAttribute('hsv').split(':');
	for (var i=0; i<3; i++) HSV[i] /= 240;
	HSV[0] *= 360;
	setBrightnessPointer(HSV[2]);
	// Adjust colors.
	var labels = bslider.firstChild.childNodes;
	for (var j=0; j<labels.length; j++) {
		HSV[2] = 1 - j/(labels.length-1);
		var RGB = HSV2RGBms(HSV);
		for (var i=0; i<3; i++) RGB[i] = Math.round(255*RGB[i]);
		labels[j].setAttribute('style','background-color: rgb(' + 
			RGB.join(',') + ');');
	}
}

function setBrightnessPointer(bright) {
	// 0<= bright <=1.
	var pointer = document.getElementById('brightnessPointer');
	var pointerHalfHeight = Math.floor(pointer.boxObject.height/2);
	var bbar = document.getElementById('brightnessSlider');
	var Bheight = bbar.boxObject.height; 
	Bheight -= 1; // Count starts at 0 not 1.
	pointer.setAttribute('top',Math.round((1-bright)*Bheight) 
		 - pointerHalfHeight);
}

function AddColor() {
	var addColorsRadiogroup = document.getElementById('adjustedColors');
	var colorBox = getNextStorage(addColorsRadiogroup) ||
		addColorsRadiogroup.firstChild.childNodes[1].firstChild.firstChild;
	var radioLabel = document.getAnonymousNodes(colorBox)[1].childNodes[1];
	var RGB = document.getElementById('rgbColors').getAttribute('rgb').split(':');
	for (var i=0; i<3; i++) RGB[i] = Math.round(RGB[i]);
	colorBox.setAttribute('value','AC' + RGB.join(':'));
	radioLabel.setAttribute('style','background-color: rgb(' + 
		RGB.join(',') + ');');
	var radioChildren = addColorsRadiogroup._getRadioChildren(); // Shame on me, use a treewalker or extend radio and radiogroup with XBL.
	for (var i=0; i< radioChildren.length-1; ++i) {
		if (radioChildren[i] == colorBox) break;
	}
	if (++i == radioChildren.length) i = 0;
	setNextStorage(addColorsRadiogroup,radioChildren[i]);
}

function getNextStorage(radiogr) {
	var colorboxes = document.getElementsByAttribute('nextStorage','true');
	//ASUME THERE IS ONLY ONE SUCH A COLORBOX, should actually check that colorbox is a child or radiogr.
	if (colorboxes.length) {
		return colorboxes[0]
	} else {
		return false;
	}
}

function setNextStorage(radiogr,radioElement) {
	var nextStorage = getNextStorage(radiogr);
	if (nextStorage) nextStorage.removeAttribute('nextStorage');
	radioElement.setAttribute('nextStorage','true');
}

function setNextStorageFromSelectedItem(evt) {
	if (evt.target.nodeName == 'radio' &&
		evt.attrName == 'selected' && evt.newValue == 'true') {
		setNextStorage(this,evt.target);
	}
}

var HSdrag = {
onDragStart: function (evt,transferData,action) {
	transferData.data = new TransferData();
	transferData.data.addDataForFlavour('text/unicode','dummy');
	document.getElementById('HScursor').setAttribute('hidden','true');
	},
getSupportedFlavours: function () {
	var flavours = new FlavourSet();
	flavours.appendFlavour("text/unicode");
	return flavours;
	},
onDragOver: function (evt,flavour,session) {
	imageClicked(evt);
	},
onDragExit: function (evt,session) {
	document.getElementById('HScursor').removeAttribute('hidden');
	putHScursor();
	}
}
var Bdrag = {
onDragStart: function (evt,transferData,action) {
	transferData.data = new TransferData();
	transferData.data.addDataForFlavour('text/unicode','dummy');
	},
getSupportedFlavours: function () {
	var flavours = new FlavourSet();
	flavours.appendFlavour('text/unicode');
	return flavours;
	},
onDragOver: function (evt,flavour,session) {
	brightnessBarClicked(evt);
	}
}
