/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is tkishdemos.
 *
 * The Initial Developer of the Original Code is
 * M.D. Nauta.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


var timer;

function initialize() {
	var editor = document.getElementById('theEditor');
	editor.makeEditable('html',false);
	applyStyleSheetToEditorContent('theEditor',
		'chrome://tkishdemos/skin/text_search_editor.css');
	setupTimerAndTree();
	document.getElementById('fileTextbox').focus();
}

function selectFile() {
	var filename = document.getElementById('fileTextbox').value;
	try {
		var aFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		aFile.initWithPath(filename);
	} catch (e) {
		alert("Error: couldn't open \"" + filename + "\": no such file or directory");
		return false;
	}
	if (!aFile.exists()) {
		alert('Error: couldn\'t open "' + filename + '": no such file or directory');
		return false;
	}
	if (!aFile.isFile() || !aFile.isReadable()) {
		alert("Error: couldn't open \"" + filename + "\": permission denied");
		return false;
	}
	removeTimer();
	var editor = document.getElementById('theEditor');
	filename = 'view-source:file://' + filename;
	editor.webNavigation.loadURI(filename,editor.webNavigation.LOAD_FLAGS_NONE,null,null,null);
	// The next two lines should only be applied after the webpage has been loaded, but how do you do this?
	applyStyleSheetToEditorContent('theEditor','chrome://tkishdemos/skin/text_search_editor.css');
	setupTimerAndTree();
	return true;
}

function applyStyleSheetToEditorContent(editorId,styleSheet) {
	var xulEditor = document.getElementById(editorId);
	alert(xulEditor); // If I remove this alert then I get trouble with
	// editingSession.editorStatus == editingSession.eCreationInProgress.
	// Don't know how to solve this.
	var editorHTML = xulEditor.getHTMLEditor(xulEditor.contentWindow);
	var editorCSS = editorHTML.QueryInterface(Components.interfaces.nsIEditorStyleSheets);
	// Make sure the URI below is available.
	editorCSS.addStyleSheet(styleSheet);
}

function highlightText() {
	var loadedDoc = document.getElementById('theEditor').contentDocument;
	var body = loadedDoc.getElementsByTagName('BODY')[0];
	walkTree(body,removeSearchTags);
	body.normalize();
	walkTree(body,insertSearchTags);
}

function walkTree(rootNode,myFunc) {
	// Need a static tree instead of dynamic, so don't use treeWalker but do it yourself.
	if (rootNode.hasChildNodes()) {
		var child = rootNode.firstChild;
		var nextChild = child.nextSibling;
		walkTree(child,myFunc);
		myFunc(child);
		while (nextChild) {
			child = nextChild;
			nextChild = child.nextSibling;
			walkTree(child,myFunc);
			myFunc(child);
		}
	}
}

function insertSearchTags(node) {
	if (node.nodeType == Node.TEXT_NODE) {
		var searchStr = document.getElementById('stringTextbox').value;
		if (searchStr.length > 0) {
			var index = node.data.indexOf(searchStr)
			if (index != -1) {
				var parent = node.parentNode;
				var tempNode = node.splitText(index);
				var secondNode = tempNode.splitText(searchStr.length);
				var searchElem = document.createElement('search');
				searchElem.appendChild(tempNode.cloneNode(false));
				parent.replaceChild(searchElem,tempNode);
				insertSearchTags(secondNode);
			}
		}
	}
}

function removeSearchTags(node) {
	if (node.nodeName == 'search') {
		var contentNodes = node.childNodes;
		var lastChild = contentNodes[contentNodes.length-1];
		var parent = node.parentNode;
		parent.replaceChild(lastChild,node);
		for (var i=0; i < contentNodes.length-1; i++) {
			parent.insertBefore(contentNodes[i],lastChild);
		}
	}
}

function setupTimerAndTree() {
	var body = document.getElementById('theEditor').contentDocument.firstChild.childNodes[1];
	var filter = function (node) {
		if (node.nodeType == Node.ELEMENT_NODE) {
			if (node.nodeName == 'search') return NodeFilter.FILTER_ACCEPT;
			return NodeFilter.FILTER_SKIP;
		}
		return NodeFilter.FILTER_REJECT;
	};
	var tw = document.createTreeWalker(body,NodeFilter.SHOW_ELEMENT,filter,false);
	var timerIf = Components.interfaces.nsITimer;
	timer = Components.classes["@mozilla.org/timer;1"].createInstance(timerIf);
	if (!timer) {
		alert("Error: couldn't create a timer.");
		return false;
	}
	var callback = {
		notify: function (timer) {
			var node;
			tw.currentNode = tw.root;
			while ((node=tw.nextNode())) {
				updateSearchHighlight(node);
			}
		}
	};
	timer.initWithCallback(callback,500,timerIf.TYPE_REPEATING_SLACK);
	return true;
}

function updateSearchHighlight(node) {
	if (node.getAttribute('highlight')) {
		node.removeAttribute('highlight');
	} else {
		node.setAttribute('highlight','true');
	}
}

function cleanup() {
	removeTimer();
}

function removeTimer() {
	if (timer) timer.cancel();
}
